@@ -16,7 +16,7 @@ use Term::ANSIColor qw/colored/;
 use File::ShareDir qw/dist_dir/;
 use Path::Tiny;
 
-our $VERSION = "0.005";
+our $VERSION = "0.006";
 
 has [qw/base_url api_key api_pass test/] => (
     is => 'rw',
@@ -36,6 +36,7 @@ has colour_map => (
     default => sub {
         my ($self) = @_;
         return {
+            '' => ['reset'],
             map {
                 ( $_ => [ split /\s+/, $self->colours->{$_} ] )
             }
@@ -43,6 +44,10 @@ has colour_map => (
         };
     },
 );
+has opt => (
+    is       => 'rw',
+    required => 1,
+);
 
 sub _jenkins {
     my ($self) = @_;
@@ -54,18 +59,30 @@ sub _jenkins {
     });
 };
 
+sub _alpha_num {
+    my $a1 = ref $a ? $a->{name} : $a;
+    my $b1 = ref $b ? $b->{name} : $b;
+    $a1 =~ s/(\d+)/sprintf "%05d", $1/egxms;
+    $b1 =~ s/(\d+)/sprintf "%05d", $1/egxms;
+    return $a1 cmp $b1;
+}
+
 sub ls { shift->list(@_) }
 sub list {
-    my ($self, $opt, $query) = @_;
+    my ($self, $query) = @_;
     my $jenkins = $self->jenkins();
 
-    $self->_action(1, $query, $self->_ls_job($opt, $jenkins));
+    if ( ! defined $self->opt->regexp ) {
+        $self->opt->regexp(1);
+    }
+
+    $self->_action(0, $query, $self->_ls_job($jenkins));
 
     return;
 }
 
 sub start {
-    my ($self, $opt, $job, @extra) = @_;
+    my ($self, $job, @extra) = @_;
     my $jenkins = $self->jenkins();
 
     _error("Must start build with job name!\n") if !$job;
@@ -75,12 +92,11 @@ sub start {
         warn "Job is not buildable!\n";
         return 1;
     }
-    if ( $result->{inQueue} && ! $opt->force ) {
+    if ( $result->{inQueue} && ! $self->opt->force ) {
         warn $result->{queueItem}{why} . "\n";
         warn "View at $result->{url}\n";
         return 0;
     }
-    die "Not yet!\n";
 
     $jenkins->trigger_build($job);
 
@@ -88,13 +104,13 @@ sub start {
 
     $result = $jenkins->_json_api(['job', $job, 'api', 'json']);
     print "View at $result->{url}\n";
-    warn Dumper $result;
+    print $result->{queueItem}{why}, "\n" if $result->{queueItem}{why};
 
     return;
 }
 
 sub delete {
-    my ($self, $opt, @jobs) = @_;
+    my ($self, @jobs) = @_;
 
     _error("Job name required for deleting jobs!\n") if !@jobs;
 
@@ -107,7 +123,7 @@ sub delete {
 }
 
 sub status {
-    my ($self, $opt, $job, @extra) = @_;
+    my ($self, $job, @extra) = @_;
     my $jenkins = $self->jenkins();
 
     _error("Job name required to show job status!\n") if !$job;
@@ -117,10 +133,10 @@ sub status {
     my $color = $self->colour_map->{$result->{color}} || [$result->{color}];
     print colored($color, $job), "\n";
 
-    if ($opt->verbose) {
+    if ($self->opt->verbose) {
         for my $build (@{ $result->{builds} }) {
             print "$build->{displayName}\t$build->{result}\t";
-            if ( $opt->verbose > 1 ) {
+            if ( $self->opt->verbose > 1 ) {
                 for my $action (@{ $build->{actions} }) {
                     if ( $action->{lastBuiltRevision} ) {
                         print $action->{lastBuiltRevision}{SHA1};
@@ -136,18 +152,26 @@ sub status {
 
 sub conf { shift->config(@_) }
 sub config {
-    my ($self, $opt, $job, @extra) = @_;
+    my ($self, $job) = @_;
     my $jenkins = $self->jenkins();
 
     _error("Must provide job name to get it's configuration!\n") if !$job;
 
-    print $jenkins->project_config($job);
+    $self->_action(0, $job, sub {
+        my $config = $jenkins->project_config($_->{name});
+        if ( $self->opt->{out} ) {
+            path($self->opt->{out}, "$_->{name}.xml")->spew($config);
+        }
+        else {
+            print $config;
+        }
+    });
 
     return;
 }
 
 sub queue {
-    my ($self, $opt, $job, @extra) = @_;
+    my ($self, $job, @extra) = @_;
     my $jenkins = $self->jenkins();
 
     my $queue = $jenkins->build_queue();
@@ -165,7 +189,7 @@ sub queue {
 }
 
 sub create {
-    my ($self, $opt, $job, $config, @extra) = @_;
+    my ($self, $job, $config, @extra) = @_;
     my $jenkins = $self->jenkins();
 
     my $success = $jenkins->create_job($job, $config);
@@ -176,7 +200,7 @@ sub create {
 }
 
 sub load {
-    my ($self, $opt, $job, $config, @extra) = @_;
+    my ($self, $job, $config, @extra) = @_;
     my $jenkins = $self->jenkins();
 
     print Dumper $jenkins->load_statistics();
@@ -185,58 +209,62 @@ sub load {
 }
 
 sub watch {
-    my ($self, $opt, @jobs) = @_;
+    my ($self, @jobs) = @_;
     my $jenkins = $self->jenkins();
 
-    $opt->{sleep} ||= 30;
+    if ( ! defined $self->opt->regexp ) {
+        $self->opt->regexp(1);
+    }
+
+    $self->opt->{sleep} ||= 30;
     my $query = join '|', @jobs;
 
     while (1) {
         my @out;
-        my $ls = $self->_ls_job($opt, $jenkins, 1);
+        my $ls = $self->_ls_job($jenkins, 1);
         print "\n...\n";
 
-        $self->_action(1, $query, sub {
+        $self->_action(0, $query, sub {
             push @out, $ls->(@_);
         });
 
         print "\e[2J\e[0;0H\e[K";
         print "Jenkins Jobs: ", (join ', ', @jobs), "\n\n";
-        print sort @out;
-        sleep $opt->{sleep};
+        print sort _alpha_num @out;
+        sleep $self->opt->{sleep};
     }
 
     return;
 }
 
 sub enable {
-    my ($self, $opt, $query) = @_;
+    my ($self, $query) = @_;
 
     my $xsl = path(dist_dir('App-JenkinsCli'), 'enable.xsl');
-    $self->_xslt_actions($opt, $query, $xsl);
+    $self->_xslt_actions($self->opt, $query, $xsl);
 
     return;
 }
 
 sub disable {
-    my ($self, $opt, $query) = @_;
+    my ($self, $query) = @_;
 
     my $xsl = path(dist_dir('App-JenkinsCli'), 'disable.xsl');
-    $self->_xslt_actions($opt, $query, $xsl);
+    $self->_xslt_actions($self->opt, $query, $xsl);
 
     return;
 }
 
 sub change {
-    my ($self, $opt, $query, $xsl) = @_;
+    my ($self, $query, $xsl) = @_;
 
-    $self->_xslt_actions($opt, $query, $xsl);
+    $self->_xslt_actions($self->opt, $query, $xsl);
 
     return;
 }
 
 sub _xslt_actions {
-    my ($self, $opt, $query, $xsl) = @_;
+    my ($self, $query, $xsl) = @_;
     require XML::LibXML;
     require XML::LibXSLT;
 
@@ -257,8 +285,8 @@ sub _xslt_actions {
         my $results = $stylesheet->transform($dom);
         my $output  = $stylesheet->output_as_bytes($results);
 
-        warn "Updating $_->{name}\n" if $opt->{verbose};
-        if ($opt->{test}) {
+        warn "Updating $_->{name}\n" if $self->opt->{verbose};
+        if ($self->opt->{test}) {
             print "$output\n";
         }
         else {
@@ -279,32 +307,54 @@ sub _action {
 
     my $data = $jenkins->_json_api([qw/api json/], { extra_params => { depth => $depth } });
 
-    for my $job (sort @{ $data->{jobs} }) {
-        next if $query && $job->{name} !~ /$query/;
+    my $re = $self->opt->regexp ? qr/$query/ : qr/\A\Q$query\E\Z/;
+
+    for my $job (sort _alpha_num @{ $data->{jobs} }) {
+        next if $query && $job->{name} !~ /$re/;
         local $_ = $job;
 
-        $action->();
+        $self->$action();
     }
 
     return;
 }
 
 sub _ls_job {
-    my ($self, $opt, $jenkins, $return) = @_;
+    my ($self, $jenkins, $return) = @_;
+    my $max = 0;
 
     return sub {
         my $name = $_->{name};
-        my $extra = '';
+        my $extra = ' ';
 
-        if ( $_->{color} =~ s/_anime// ) {
+        if ( ! $_->{color} ) {
+            $_->{color} = '';
+        }
+        elsif ( $_->{color} =~ s/_anime// ) {
             $extra = '*';
         }
 
-        if ( $opt->{verbose} ) {
+        if ( $self->opt->{verbose} ) {
             eval {
-                my $details = $jenkins->_json_api(['job', $_->{name}, qw/api json/], { extra_params => { depth => 1 } });
+                my $details = $jenkins->_json_api(
+                    ['job', $_->{name}, qw/api json/],
+                    {
+                        extra_params => {
+                            depth => 0,
+                            tree => 'lastBuild[timestamp,displayName,builtOn]'
+                        }
+                    }
+                );
                 $extra .= "\t" . localtime( ( $details->{lastBuild}{timestamp} || 0 ) / 1000 );
-                $extra .= "\t($details->{lastBuild}{displayName} / $details->{lastBuild}{builtOn})";
+                if ( $details->{lastBuild}{displayName} && $details->{lastBuild}{builtOn} ) {
+                    $extra .= "\t($details->{lastBuild}{displayName} / $details->{lastBuild}{builtOn})";
+                }
+                else {
+                    $extra .= "\tNever run";
+                }
+                1;
+            } or do {
+                warn "Error getting job $_->{name}'s details: $@\n";
             };
             $name = $self->base_url . 'job/' . $name;
         }
@@ -312,7 +362,15 @@ sub _ls_job {
         # map "jenkins" colours to real colours
         my $color = $self->colour_map->{$_->{color}} || [$_->{color}];
 
-        my $out = colored($color, $name) . " $extra\n";
+        if ( length $name > $max ) {
+            $max = length $name;
+        }
+
+        my $out = colored($color, sprintf "% -${max}s", $name) . " $extra\n";
+
+        if ( $self->opt->{long} ) {
+            $out = "$_->{color} $out";
+        }
 
         if ($return) {
             return $out;
@@ -331,7 +389,7 @@ App::JenkinsCli - Command line tool for interacting with Jenkins
 
 =head1 VERSION
 
-This documentation refers to App::JenkinsCli version 0.005
+This documentation refers to App::JenkinsCli version 0.006
 
 =head1 SYNOPSIS
 
@@ -346,55 +404,59 @@ This documentation refers to App::JenkinsCli version 0.005
 
 =head1 SUBROUTINES/METHODS
 
-=head2 C<ls ($opt, $query)>
+=head2 C<ls ($query)>
 
-=head2 C<list ($opt, $query)>
+=head2 C<list ($query)>
 
 List all jobs, optionally filtering with C<$query>
 
-=head2 C<start ($opt, $job)>
+=head2 C<start ($job)>
 
 Start C<$job>
 
-=head2 C<delete ($opt, $job)>
+=head2 C<delete ($job)>
 
 Delete C<$job>
 
-=head2 C<status ($opt, $job)>
+=head2 C<status ($job)>
 
 Status of C<$job>
 
-=head2 C<enable ($opt, $job)>
+=head2 C<enable ($job)>
 
 enable C<$job>
 
-=head2 C<disable ($opt, $job)>
+=head2 C<disable ($job)>
 
 disable C<$job>
 
-=head2 C<conf ($opt, $job)>
+=head2 C<conf ($job)>
 
-=head2 C<config ($opt, $job)>
+=head2 C<config ($job)>
 
 Show the config of C<$job>
 
-=head2 C<queue ($opt)>
+=head2 C<queue ()>
 
 Show the queue of running jobs
 
-=head2 C<create ($opt, $job)>
+=head2 C<create ($job)>
 
 Create a new Jenkins job
 
-=head2 C<load ($opt)>
+=head2 C<load ()>
 
 Show the load stats for the server
 
-=head2 C<change ($opt, $query, $xsl)>
+=head2 C<change ($query, $xsl)>
 
 Run the XSLT file (C<$xsl>) over each job matching C<$query> to generate a
 new config which is then sent back to Jenkins.
 
+=head2 C<watch ($job)>
+
+Watch jobs to track changes.
+
 =head1 ATTRIBUTES
 
 =over 4
@@ -423,6 +485,10 @@ Internal L<Jenkins::API> object
 
 Mapping of Jenkins states to L<Term::ANSIColor>s
 
+=item opt
+
+User options
+
 =back
 
 =head1 DIAGNOSTICS
